@@ -36,7 +36,7 @@ use Dancer::Plugin::Database;
 use HTML::Table::FromDatabase;
 use CGI::FormBuilder;
 
-our $VERSION = '0.05';
+our $VERSION = '0.10';
 
 =head1 NAME
 
@@ -81,7 +81,8 @@ L<HTML::Table::FromDatabase> to display lists of records.
             weight => qr/\d+/,
         },
         key_column => 'sku',
-        editable => [ qw( f_name l_name adr_1 ),
+        editable_columns => [ qw( f_name l_name adr_1 ),
+        display_columns => [ qw( f_name l_name adr_1 ),
         deleteable => 1,
     );
 
@@ -157,12 +158,12 @@ acceptable values, for instance:
   { foo => [ qw(Foo Bar Baz) ] }
 
 
-=item C<editable> (optional)
+=item C<editable_columns> (optional)
 
 Specify an arrayref of fields which the user can edit.  By default, this is all
 columns in the database table, with the exception of the key column.
 
-=item <not_editable> (optional)
+=item <not_editable_columns> (optional)
 
 Specify an arrayref of fields which should not be editable.
 
@@ -179,6 +180,10 @@ Specify whether to support deleting records.  If set to a true value, a route
 will be created for C</prefix/delete/:id> to delete the record with the ID
 given, and the edit form will have a "Delete $record_title" button.
 
+=item C<display_columns>
+
+Specify an arrayref of columns that should show up in the list.  Defaults to all.
+
 =cut
 
 sub simple_crud {
@@ -303,6 +308,12 @@ sub _create_add_edit_route {
 	    map { $_->{COLUMN_NAME} } @$all_table_columns;
     }
 
+    if ($args->{not_editable_columns}) {
+        for my $col (@{$args->{not_editable_columns}}) {
+            @editable_columns = grep { $_ ne $col } @editable_columns;
+        }
+    }
+
     # Some DWIMery: if we don't have a validation rule specified for a
     # field, and it's pretty clear what it is supposed to be, just do it:
     my $validation = $args->{validation} || {};
@@ -438,11 +449,31 @@ sub _create_list_handler {
 
     my $dbh     = database($args->{db_connection_name});
     my $columns = _find_columns($dbh, $table_name);
+
+    my $display_columns = $args->{'display_columns'};
+
+    # If display_columns argument was passed, filter the column list to only
+    # have the ones we asked for.
+    if(ref $display_columns eq 'ARRAY') {
+        my @filtered_columns;
+
+        foreach my $col (@$columns) {
+            if(grep { $_ eq $col->{'COLUMN_NAME'} } @$display_columns) {
+                push @filtered_columns, $col;
+            }
+        }
+
+        if(@filtered_columns) {
+            $columns = \@filtered_columns;
+        }
+    }
+
     my $options = join(
 	"\n",
 	map { "<option value='$_->{COLUMN_NAME}'>$_->{COLUMN_NAME}</option>" }
 	    @$columns
     );
+
     my $html = <<"SEARCHFORM";
  <p><form name="searchform" method="get">
      Field:  <select name="searchfield">$options</select> &nbsp;&nbsp;
@@ -457,7 +488,13 @@ SEARCHFORM
 	INT     => q{%s = %s},
 	VARCHAR => q{%s LIKE %s},
     );
-    my $query = "SELECT *, $key_column AS actions FROM $table_name";
+
+    # Explicitly select the columns we are displaying.  (May have been filtered
+    # by display_columns above.)
+    my $select_cols = join(',', 
+        map { database->quote_identifier($_->{'COLUMN_NAME'}) } @$columns);
+
+    my $query = "SELECT $select_cols, $key_column AS actions FROM $table_name";
 
     if (params->{'q'}) {
 	my ($column_data)
@@ -682,10 +719,12 @@ Alberto Simões (ambs)
 
 Jonathan Barber
 
+saberworks
+
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2010 David Precious.
+Copyright 2010-11 David Precious.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
